/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global watch, data*/

(function() {
    /**
     * Save data in the colorCity array into the local storage
     * @private
     */
    function setLocalStorageData() {
        var jsonString = JSON.stringify(data.colorCity);

        localStorage.setItem("colorCity", jsonString);
    }

    /**
     * Get data from the local storage and save them into the colorCity array
     * @private
     */
    function getLocalStorageData() {
        var i,
            jsonObj = JSON.parse(localStorage.getItem("colorCity"));

        // Executed except when there is no data saved in the local storage
        if (jsonObj !== null) {
            for (i = 0; i < data.colorCity.length; i++) {
                data.colorCity[i] = jsonObj[i];
            }
        }
    }

    /**
     * Checks the feasibility of the selected value
     * If possible, set the selected value as new data for colorCity array
     * @private
     */
    function setColorCity(event) {
        var j,
            isSelected = 0,
            colorId = 0,
            cityId = 0,
            thisCity = event.target;

        colorId = thisCity.id.replace("colorCity-", "");
        cityId = thisCity.value;

        // Check if the city has already been selected (except for "None")
        if (cityId !== "0") {
            for (j = 0; j < data.colorCity.length; j++) {
                if (cityId === data.colorCity[j].cityId) {
                    isSelected = 1;
                    break;
                }
            }
        }

        // If the city has already been selected, display an alert message
        // Otherwise, change the data in colorCity array with the new value
        if (isSelected === 1) {
            alert("City Already Selected!");
            thisCity.value = data.colorCity[colorId].cityId;
        } else if (isSelected === 0) {
            data.colorCity[colorId].cityId = thisCity.value;
        }
    }

    /**
     * Makes the select table dynamically
     * @private
     */
    function makeSelectTable() {
        var i,
            j,
            div = null,
            canvas = null,
            select = null,
            option = null,
            divTable = document.querySelector("#select-table"),
            optionFragment = document.createDocumentFragment(),
            selectFragment = document.createDocumentFragment(),
            divFragment = document.createDocumentFragment(),
            centerY = document.body.clientHeight / 2,
            TABLE_HEIGHT = 300;

        // Set the size of the table accordingly with the size of the screen
        divTable.style.paddingTop = (centerY - (TABLE_HEIGHT / 2)) + "px";

        for (i = 0; i < data.colorCity.length; i++) {
            div = document.createElement("div");
            div.setAttribute("align", "center");
            canvas = document.createElement("canvas");
            canvas.id = "color-" + i;
            canvas.style.verticalAlign = "middle";
            canvas.style.backgroundColor = "#" + data.colorCity[i].colorCode;
            canvas.style.marginRight = "5px";
            canvas.width = 40;
            canvas.height = 40;
            select = document.createElement("select");
            select.className = "selectCity";
            select.id = "colorCity-" + i;
            select.style.fontSize = "2em";
            select.style.verticalAlign = "middle";
            for (j = 0; j < data.cityArray.length; j++) {
                option = document.createElement("option");
                option.text = data.cityArray[j].cityName;
                option.value = j;
                optionFragment.appendChild(option);
            }
            select.appendChild(optionFragment);
            selectFragment.appendChild(canvas);
            selectFragment.appendChild(select);
            div.appendChild(selectFragment);
            divFragment.appendChild(div);
        }
        divTable.appendChild(divFragment);

        // Every time the value of the select element changes, run setColorCity function to set new data
        for (i = 0; i < data.colorCity.length; i++) {
            document.querySelectorAll(".selectCity")[i].addEventListener("change", setColorCity);
        }
    }

    /**
     * Changes between the watch face page and the select table page
     * @param {string} page - the name of the page to be displayed ("watch" or "city")
     * @private
     */
    function changePage(page) {
        if (page === "watch") {
            watch.drawWatch();

            document.querySelector("#page-watch").style.display = "block";
            document.querySelector("#page-city").style.display = "none";
        } else if (page === "city") {
            var i;

            for (i = 0; i < data.colorCity.length; i++) {
                document.querySelectorAll(".selectCity")[i].value = data.colorCity[i].cityId;
            }

            document.querySelector("#page-watch").style.display = "none";
            document.querySelector("#page-city").style.display = "block";
        }
    }

    /**
     * Sets default event listeners
     * @private
     */
    function setDefaultEvents() {
        document.querySelector("#canvas-watch").addEventListener("touchstart", function(e) {
            var touchList = e.touches;

            if (touchList.length === 2) {
                changePage("city");
            }
        });

        // When the save button is clicked, save the data in the array into the local storage
        document.querySelector("#btn-save").addEventListener("click", function() {
            setLocalStorageData();
            changePage("watch");
        });

        // When the cancel button is clicked, discard the change
        // by reloading the data in the array with the data saved previously in the local storage
        document.querySelector("#btn-cancel").addEventListener("click", function() {
            getLocalStorageData();
            changePage("watch");
        });

        // Add eventListener for tizenhwkey
        window.addEventListener("tizenhwkey", function(e) {
            if (e.keyName === "back") {
                // If it is currently on the page for selecting cities, do the same as clicking the cancel button
                // Otherwise, exit the application
                if (document.querySelector("#page-city").style.display === "block") {
                    getLocalStorageData();
                    changePage("watch");
                } else {
                    try {
                        tizen.application.getCurrentApplication().exit();
                    } catch (error) {}
                }
            }
        });
    }

    /**
     * Initiates the application
     * @private
     */
    function init() {
        watch.setDefaultVariables();
        setDefaultEvents();

        // Load data from the local storage when starting the application and make the select table accordingly
        getLocalStorageData();
        makeSelectTable();

        // Update the content of the watch every half second
        setInterval(function() {
            watch.drawWatch();
        }, 500);
    }

    window.onload = init;
}());